package de.ozgcloud.formcycle;

import static org.assertj.core.api.Assertions.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullSource;
import org.junit.jupiter.params.provider.ValueSource;

class HtmlUtilsTest {

	@Nested
	@DisplayName("Remove HTML tags")
	class TestRemoveHtml {

		@DisplayName("should return")
		@ParameterizedTest(name = "empty string for \"{0}\"")
		@NullSource
		@ValueSource(strings = { " ", "" })
		void shouldReturnEmpty(String input) {
			var actual = HtmlUtils.removeHtmlTags(input);

			assertThat(actual).isEmpty();
		}

		@Test
		@DisplayName("should remove html tags from text")
		void shouldRemoveHtml() {
			var withHtml = "This is a <b>test</b> with <i>html</i>.";
			var expected = "This is a test with html.";

			var actual = HtmlUtils.removeHtmlTags(withHtml);

			assertThat(actual).isEqualTo(expected);
		}

		@Test
		void shouldHandleEmptyAfterCleanup() {
			var cleanedText = HtmlUtils.removeHtmlTags("<b></b>");

			assertThat(cleanedText).isEqualTo(StringUtils.EMPTY);
		}
	}

	@Nested
	class TestRemoveEmptyLines {

		@DisplayName("should return")
		@ParameterizedTest(name = "empty string for \"{0}\"")
		@NullSource
		@ValueSource(strings = { " ", "", "\n", "\r", "\r\n" })
		void shouldReturnEmpty(String input) {
			var actual = HtmlUtils.removeEmptyLines(input);

			assertThat(actual).isEmpty();
		}

		@Test
		@DisplayName("should remove empty lines")
		void testRemoveEmptyLines() {
			var expectedText = "text";
			var withEmptyLines = "\n" + "\r" + "\r\n" + expectedText;

			var resultText = HtmlUtils.removeEmptyLines(withEmptyLines);

			assertThat(resultText).isEqualTo(expectedText);
		}
	}
}