package de.ozgcloud.formcycle;

import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Spy;

import de.xima.fc.interfaces.plugin.lifecycle.IPluginInitializeData;
import de.xima.fc.interfaces.workflow.value.IRecordValueDescriptorBuilder;
import de.xima.fc.interfaces.workflow.value.IStringValueDescriptor;
import de.xima.fc.interfaces.workflow.value.IValueDescriptor;
import de.xima.fc.interfaces.workflow.value.IValueDescriptorFactory;

class WorkflowElementNodePluginTest {

	@Spy
	private WorkflowElementNodePlugin plugin = new WorkflowElementNodePlugin();

	@Nested
	class TestInitialization {

		@Mock
		private IPluginInitializeData initializeData;
		@Mock
		private PluginPropertiesMapper propertiesMapper;

		@BeforeEach
		void setup() {
			when(plugin.createPropertiesMapper()).thenReturn(propertiesMapper);
		}

		@Test
		void shouldCallCreatePropertiesMapper() {
			plugin.initialize(initializeData);

			verify(plugin).createPropertiesMapper();
		}

		@Test
		void shouldCallMapProperties() {
			var properties = PropertiesTestFactory.create();
			when(initializeData.getProperties()).thenReturn(properties);

			plugin.initialize(initializeData);

			verify(propertiesMapper).mapProperties(properties);
		}
	}

	@Nested
	@DisplayName("Success value descriptor")
	class TestSuccessValueDescriptor {

		@Mock
		private IValueDescriptorFactory descriptorFactory;
		@Mock
		private IRecordValueDescriptorBuilder descriptorBuilder;
		@Mock
		private IStringValueDescriptor stringValueDescriptor;

		@BeforeEach
		void setup() {
			when(descriptorFactory.recordBuilder()).thenReturn(descriptorBuilder);
			when(descriptorBuilder.requiredProperty(any(), any(IValueDescriptor.class))).thenReturn(descriptorBuilder);
		}

		@Test
		@DisplayName("should has vorgangnummer property")
		void shouldAddVorgangnummerProperty() {
			when(descriptorFactory.string()).thenReturn(stringValueDescriptor);

			plugin.getSuccessValueDescriptor(descriptorFactory);

			verify(descriptorBuilder).requiredProperty(ExecutionResult.VORGANGNUMMER_PROPERTY_KEY, stringValueDescriptor);
		}
	}
}